searchNodes=[{"type":"module","title":"Midiex","doc":"This is the main Midiex module. It's built around three basic concepts: Ports: list or count MIDI ports availble (for example, a keyboard or synth) Connections: open or close connections to MIDI ports create a virtual input or output connections so your Elixir application appears as a MIDI device Messages: send or receive messages to and from connections. Examples # List MIDI ports Midiex . ports ( ) # Lists MIDI ports discoverable on your system # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;IAC Driver Bus 1&quot;, # num: 0, # port_ref: #Reference&lt;0.2239960018.1937899544.176288&gt; # }, # %Midiex.MidiPort{ # direction: :output, # name: &quot;IAC Driver Bus 1&quot;, # num: 0, # port_ref: #Reference&lt;0.2239960018.1937899544.176289&gt; # } # ] # Create a virtual output connection piano = Midiex . create_virtual_output_conn ( &quot;piano&quot; ) # Returns an output connection: # %Midiex.OutConn{ # conn_ref: #Reference&lt;0.1633267383.3718381569.210768&gt;, # name: &quot;piano&quot;, # port_num: 0 # } # Send MIDI messages to a connection # In the message below, the note 60 is equivalent to Middle C and 127 means maximum velocity note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off ) Livebook tour Also see the introductory tour in LiveBook at /livebook/midiex_notebook.livemd .","ref":"Midiex.html"},{"type":"function","title":"Midiex.close/1","doc":"Closes a MIDI output connection. Accepts as the first parameter either a: MIDI output connection, e.g. a %Midiex.OutConn{} struct List of output connections. Example # Connect to the first output port out_port = Midiex . ports ( :output ) |&gt; List . first ( ) out_conn = Midiex . open ( out_port ) Midiex . close_out_conn ( out_conn ) # Will return :ok if successful","ref":"Midiex.html#close/1"},{"type":"function","title":"Midiex.create_virtual_input/1","doc":"Creates a virtual input port struct. Takes a name as the first parameter. This is only available on platforms that support virtual ports (currently every platform but Windows). Important Even though this creates an input port, beacause it's a virtual port it is listed as an 'output' when querying the OS for available devices. That means other software or devices will discover it and use it as a an output and can send messages to it. It also means it will show as %Midiex.MidiPort{direction: :output} when calling Midiex.ports() . Note that it won't be discoverable until the input port is subscribed. Example # Create a virtual MIDI input by giving it a name. MIDIex will also assign it an input port number (`num`). my_virtual_in = Midiex . create_virtual_input ( &quot;My Virtual Input&quot; ) # This will return a VirtualMidiPort struct in the following format # %Midiex.VirtualMidiPort{direction: :input, name: &quot;My Virtual Input&quot;, num: 1} The %Midiex.VirtualMidiPort{} struct can then be passed to MIDI input port listener functions, such as: Midiex.subscribe(my_virtual_in) If using a Listener GenServer: Midiex.Listener.start(port: my_virtual_in) Midiex.Listener.subscribe(listener, my_virtual_in) Likewise, once subscribed to, the virtual input port can be unsubscribed to: Midiex.unsubscribe(my_virtual_in) If using a Listener GenServer: Midiex.Listener.unsubscribe(my_virtual_in)","ref":"Midiex.html#create_virtual_input/1"},{"type":"function","title":"Midiex.create_virtual_output/1","doc":"Creates a virtual output connection. This allows your Elixir application to be seen as a MIDI device. Note this is only available on platforms that support virtual ports (currently every platform but Windows). # Create an output connection called &quot;piano&quot; piano_conn = Midiex . create_virtual_output ( &quot;piano&quot; ) You can send messages to MIDI software or hardware connected to this virtual device in the standard way, e.g.: note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off ) Important Even though this creates an output port, beacause it's a virtual port it is listed as an 'input' when querying the OS for available devices. That means other software or devices will discover it and use it as a an input, such as to receive messages. It also means it will show as %Midiex.MidiPort{direction: :input} when calling Midiex.ports() .","ref":"Midiex.html#create_virtual_output/1"},{"type":"function","title":"Midiex.open/1","doc":"Creates a connection to the MIDI port. Accepts one of the following as a parameter: MIDI output port, e.g. a %Midiex.MidiPort{direction: :output} struct List of MIDI output ports. Returns an output connection ( %Midiex.OutConn{) ) or a list of output connections if a list was output ports was given as the first parameter. Example Connect to a single output port # Get the first available MIDI output port out_port = Midiex . ports ( :output ) |&gt; List . first ( ) # Open an output connection out_conn = Midiex . open ( out_port ) # Returns an output connection struct, for example: % Midiex.OutConn { conn_ref : # Reference &lt; 0.3613027763 . 2067398660.163505 &gt; , name : &quot;IAC Driver Bus 1&quot; , port_num : 0 } You can now send messages to the connection: # Send a note on message for D4 Midiex . msg_send ( out_conn , Midiex.Message . note_on ( :D4 ) ) # Let note play for 2 seconds :timer . sleep ( 2000 ) # Send a note off message for D4 Midiex . msg_send ( out_conn , Midiex.Message . note_off ( :D4 ) ) Connect to a list of output ports # Get a list of available output ports out_ports = Midiex . ports ( :output ) # Returns a list of available MIDI output ports, e.g.: [ % Midiex.MidiPort { direction : :output , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58431 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58433 &gt; } ] # Connect to each port out_conns = Midiex . open ( out_ports ) # Returns a list of MIDI output connections, e.g.: [ % Midiex.OutConn { conn_ref : # Reference &lt; 0.1633267383 . 3718381569.210768 &gt; , name : &quot;Arturia MicroFreak&quot; , port_num : 1 } , % Midiex.OutConn { conn_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; , name : &quot;KeyStep Pro&quot; , port_num : 2 } , % Midiex.OutConn { conn_ref : # Reference &lt; 0.3139841870 . 4103995416.58433 &gt; , name : &quot;MiniFuse 2&quot; , port_num : 3 } ]","ref":"Midiex.html#open/1"},{"type":"function","title":"Midiex.port_count/0","doc":"Returns the count of the number of input and output MIDI ports in as a map. Midiex . port_count ( ) # Returns a map in the following format: # %{input: 2, output: 0}","ref":"Midiex.html#port_count/0"},{"type":"function","title":"Midiex.ports/0","doc":"Lists MIDI ports availabile on the system. Midiex . ports ( ) # Returns a list of input or output ports: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Piano&quot;, # num: 0, # port_ref: #Reference&lt;0.249304305.242352152.40090&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;Drums&quot;, # num: 1, # port_ref: #Reference&lt;0.249304305.242352152.40091&gt; # } # ]","ref":"Midiex.html#ports/0"},{"type":"function","title":"Midiex.ports/1","doc":"List MIDI ports matching the specified direction (e.g. input or output) Takes an atom as the first parameter representing the direction: :input - lists input ports only :output - lists output ports only. Midiex . ports ( :input ) # Returns a list of input or output MIDI ports: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Piano&quot;, # num: 0, # port_ref: #Reference&lt;0.249304305.242352152.40090&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;Drums&quot;, # num: 1, # port_ref: #Reference&lt;0.249304305.242352152.40091&gt; # } # ]","ref":"Midiex.html#ports/1"},{"type":"function","title":"Midiex.ports/2","doc":"Lists MIDI ports matching the name. This can be either a string match or a regex pattern. Optionally takes a direction (:input or :output) can be given. Examples: # Regex examples # List ports containing the word &#39;Arturia&#39; and ignore case Midiex . ports ( ~r/Arturia/i ) # List output ports starting the word &#39;Arturia&#39; and ignore case Midiex . ports ( ~r/^Arturia/i , :output ) # String matching examples # List input ports with the name &#39;KeyStep Pro&#39; Midiex . ports ( &quot;KeyStep Pro&quot; , :input ) # List output ports with the name &#39;Arturia MicroFreak&#39; Midiex . ports ( &quot;Arturia MicroFreak&quot; , :output )","ref":"Midiex.html#ports/2"},{"type":"function","title":"Midiex.send_msg/2","doc":"Sends a binary MIDI message to one or more output connection(s). Takes the following parameters: Output connection: which is an %Midiex.OutConn{} struct or a list of Midiex.OutConn{} structs MIDI message: which is in a binary format, such as &lt;&lt;0x90, 60, 127&gt;&gt; Returns the same output connection or a list of output connections passed to it. This is so you can chain messages together. Example Send a message to single output connection # Connect to the first available MIDI output port out_conn = Midiex . ports ( :output ) |&gt; List . first ( ) |&gt; Midiex . open ( ) # Play the note D3 on all that connection for 3 seconds out_conn |&gt; Midiex . send_msg ( Midiex.Message . note_on ( :D3 ) ) |&gt; tap ( fn _ -&gt; :timer . sleep ( 3000 ) end ) # wait 3 seconds |&gt; Midiex . send_msg ( Midiex.Message . note_off ( :D3 ) ) Send the same message to ALL available outputs # Get a list of output ports and create connections for them out_conns = Midiex . ports ( :output ) |&gt; Midiex . open ( ) # Play the note D3 on all those connections for 3 seconds out_conns |&gt; Midiex . send_msg ( Midiex.Message . note_on ( :D3 ) ) |&gt; tap ( fn _ -&gt; :timer . sleep ( 3000 ) end ) # wait 3 seconds |&gt; Midiex . send_msg ( Midiex.Message . note_off ( :D3 ) )","ref":"Midiex.html#send_msg/2"},{"type":"function","title":"Midiex.subscribe/1","doc":"Low-level API for subscribing to one or more MIDI input ports. This includes ports created via create_virtual_input/1 . The first parameter accepts either: A single %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} struct A list of %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} structs. The calling process will receive MIDI messages from the ports subscribed to. The source of the message will be undifferentiated, so consider using a different calling process for different inputs if they need to be handled separately. Example # Get a list of MIDI input ports on the system midi_input_ports = Midiex . ports ( :input ) # This function will return a list of ports discovered, e.g.: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Arturia MicroFreak&quot;, # num: 1, # port_ref: #Reference&lt;0.3704955737.2291269656.36760&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;KeyStep Pro&quot;, # num: 2, # port_ref: #Reference&lt;0.3704955737.2291269656.36761&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;MiniFuse 2&quot;, # num: 3, # port_ref: #Reference&lt;0.3704955737.2291269656.36762&gt; # } # ] # Subscribe to the input ports. The current process will receive MIDI messages. Returns `:ok` if successful. Midiex . subscribe ( midi_input_ports ) You'll need to implement message recieving in your process. Alternative: use a Listener process As an alterantive you can use Midiex.Listener GenServer which subscribes to MIDI input ports and forwards any messages received to event handlers. For the above example, if you wanted to inspect messages coming in on those ports, you could: alias Midiex.Listener # Start a lister for this MIDI input port { :ok , listner } = Listener . start ( port : midi_input_ports ) # Create a handler than inspects the MIDI messages received: Listener . add_handler ( listener , fn ( midi_msg ) -&gt; IO . inspect ( midi_msg , label : &quot;Msg received&quot; ) end ) # Any messages received will be inspected on the console, e.g.: # Msg received: [130, 84, 0] # Msg received: [146, 60, 43] # Msg received: [130, 60, 0] # Msg received: [146, 72, 34] # Msg received: [130, 72, 0] # Msg received: [146, 76, 40] # Msg received: [130, 76, 0] # Msg received: [146, 64, 47] # Msg received: [130, 64, 0] # Msg received: [146, 84, 30]","ref":"Midiex.html#subscribe/1"},{"type":"function","title":"Midiex.subscribed_ports/0","doc":"Returns a list of ports currently subscribed to using the subscribe/1 function. This will include any vitual input ports currently subscribed to. Example Midiex . subscribed_ports ( ) # Returns of list of subscribed ports, including virtual ports, e.g.: [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.3977715800 . 1277296664.256043 &gt; } , % Midiex.VirtualMidiPort { direction : :input , name : &quot;My Virtual Input&quot; , num : 1 } ]","ref":"Midiex.html#subscribed_ports/0"},{"type":"function","title":"Midiex.unsubscribe/1","doc":"Unsubscribes from recieving MIDI messages from an input port connection. Messages stop as this function releases the OS thread created for listening to the input conection. This function takes as the first parameter one of the following: Port struct: A MIDI input port %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} struct List: A list of MIDI input port %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} structs Number: A MIDI input port number (this is the integer in the :num key within the %Midiex.MidiPort{} ) (non-virtual ports only) Atom: The atom :all , which will unsubscribe from all MIDI input ports subscribed to, including virtual ports. If you would like to unsubscribe to virtual ports or ones listed on your device by the OS only, use unsubscribe(:all, :virtual) or unsubscribe(:all, :device) instead. Example # Unsubscribe from all inport ports listed on your device&#39;s OS Midiex . unsubscribe ( :all , :device ) # Unsubscribe from all virtual ports you created Midiex . unsubscribe ( :all , :virtual )","ref":"Midiex.html#unsubscribe/1"},{"type":"module","title":"Midiex.Listener","doc":"GenServer for subscribing and responding to MIDI input ports. This GenServer works by: Subscribing to one or more MIDI input ports (using Midiex.subscribe/1 ). For each MIDI input port, Midiex.subscribe/1 will create a new OS thread (in Rust) which establishes a connection to the port and listens to messages. Incoming messages are then forwarded to the calling Elixir process (in this case, the Midiex.Listener process.) A subscription can be established on the start/1 or subscribe/2 functions, e.g.: # Get the first MIDI input port input_port = Midiex . ports ( :input ) |&gt; List . first ( ) # Start a lister for this MIDI input port { :ok , listner } = Midiex.Listener . start ( port : input_port ) Receieves MIDI messages and passes it onto one or more Elixir handler functions. The handler takes one parameter representing the MIDI message, e.g.: # Add a simple message handler which inspects each message received: Listener . add_handler ( listener , fn ( midi_msg ) -&gt; IO . inspect ( midi_msg ) end ) Example alias Midiex.Listener # Get the first MIDI input port input_port = Midiex . ports ( :input ) |&gt; List . first ( ) # Start a lister for this MIDI input port { :ok , listner } = Listener . start ( port : input_port ) # Create a handler than inspects the MIDI messages received: my_msg_hander = fn ( midi_msg ) -&gt; IO . inspect ( midi_msg , label : &quot;MIDI message&quot; ) end Listener . add_handler ( listener , &amp; my_msg_hander / 1 ) # Stop listening to the input port Listener . unsubscribe ( listner , input_port )","ref":"Midiex.Listener.html"},{"type":"function","title":"Midiex.Listener.add_handler/2","doc":"Add one or more callback function(s) which will recieve and handle MIDI messages. A single callback function or multiple callback functions can be provided in a list. Example # Start your Listener process { :ok , listener } = Listener . start ( port : input_port ) # Add a single handler Listener . add_handler ( listener , fn msg -&gt; IO . inspect msg , label : &quot;Inspecting msg&quot; end ) # Add multiple handlers in a list Listener . add_handler ( listener , [ fn msg -&gt; IO . inspect msg , label : &quot;Msg handler 1&quot; end , fn msg -&gt; IO . inspect msg , label : &quot;Msg handler 1&quot; end , ] ) # If you&#39;ve defined your hander function in a module function, pass it the usual way: Listener . add_handler ( listener , &amp; MyModule . function_name / 1 )","ref":"Midiex.Listener.html#add_handler/2"},{"type":"function","title":"Midiex.Listener.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Midiex.Listener.html#child_spec/1"},{"type":"function","title":"Midiex.Listener.get_state/1","doc":"Gets the servers state, returns %Midiex.Listener{} struct.","ref":"Midiex.Listener.html#get_state/1"},{"type":"function","title":"Midiex.Listener.new/1","doc":"Creates a new %Midiex.Server{} struct. Takes an optional keyword list as the first parameter which can be used to populate individual struct keys. The struct holds the following key-values: :port which holds a list of MIDI input ports to listen to. These can be device ports %Midiex.MidiPort{direction: :input} or virtual ports %Midiex.VirtualMidiPort{} :callback which holds a list of functions called when a message is received for an input port. The callback must be of single arity and take it's first parameter a message. See add_handler/2 for an example.","ref":"Midiex.Listener.html#new/1"},{"type":"function","title":"Midiex.Listener.start/1","doc":"Start the Midiex.Server GenServer. Takes an optional keyword list as the first parameter which can be used to populate individual %Midiex.Listener{} struct keys. See new/1 for informaton. Examples # Start with no options { :ok , listener } = Midiex.Listener . start ( ) # Start, already passing the first available input port to listen to first_port = Midiex . ports ( :input ) |&gt; List . first ( ) { :ok , listener } = Midiex.Listener . start ( ports : first_port ) # Start, already passing a list of all input ports available to listen to { :ok , listener } = Midiex.Listener . start ( ports : Midiex . ports ( :input ) )","ref":"Midiex.Listener.html#start/1"},{"type":"function","title":"Midiex.Listener.subscribe/2","doc":"Subscribe to one or more MIDI input ports. This accepts both ports listed on your device %Midiex.MidiPort{direction: :input} and virtual ports %Midiex.VirtualMidiPort{} you've created. It accepts as it's second parameter either: a single MIDI input port a list of MIDI input ports :all atom which will stop all MIDI input ports subscribed to. Example # Subscribe to the input port of the Arturia KeyStep Pro keyboard keystep_in_port = Midiex . port ( &quot;KeyStep Pro&quot; , :input ) # Returns a list with matching port names, in this case: [ % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; } ] # Create and start a listener process { :ok , keyboard } = Midiex.Listener . start ( ) # Listen to MIDI messages from the keyboard Midiex.Listener . subscribe ( keyboard , keystep_in_port ) # Any keys you push on the keyboard will be listened to. Add one or more handlers with Midiex.Listener.add_handler/2 to process messages.","ref":"Midiex.Listener.html#subscribe/2"},{"type":"function","title":"Midiex.Listener.unsubscribe/2","doc":"Stops listening to the MIDI input port by unsubscribing to it. This accepts both ports listed on your device %Midiex.MidiPort{direction: :input} and virtual ports %Midiex.VirtualMidiPort{} you've created. It accepts as it's second parameter either: a single MIDI input port a list of MIDI input ports :all atom which will stop all MIDI input ports subscribed to. Important This stops the Rust OS thread from sending messages from that MIDI input port. If other Elixir processes have also subscribed to that port, they will also stop recieving messages.","ref":"Midiex.Listener.html#unsubscribe/2"},{"type":"module","title":"Midiex.Message","doc":"Conveniences for creating MIDI messages. About MIDI messages MIDI messages are in the format of: status_byte + note_number (0-127) + velocity (0-127) For example, taking the status byte for Note On which in HEX is 0x90 , and the note Middle C which is 60 and a maximum key velocity of 127, the MIDI message in binary format is: &lt;&lt;0x90, 60, 127&gt;&gt; MIDI message functions So that you don't have to remember all the MIDI message codes, this library has the following functions to generate messages: note_on(note_number, velocity, opts) note_off(note_number, velocity, opts) polyphonic_aftertouch(note_number, pressure, opts) channel_aftertouch(note_number, pressure, opts) control_change(control_number, value, opts) program_change(program_number, opts) pitch_wheel(lsbyte, msbyte, opts) sysex - coming soon","ref":"Midiex.Message.html"},{"type":"function","title":"Midiex.Message.change_sound_bank/2","doc":"","ref":"Midiex.Message.html#change_sound_bank/2"},{"type":"function","title":"Midiex.Message.control_change/2","doc":"Creates a MIDI CC or 'control change' message. The following options can be passed: value: depends on the control function, but usually is a a number between 0 and 127. See the MIDI 1.0 Control Change Messages Spec or consult the MIDI device manual for specific codes an values. channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example The MIDI CC message of 123 equates to &quot;All Notes Off&quot;, thus stopping all notes being played. # Create a &#39;all notes off&#39; CC message. Midiex . control_change ( 127 ) Reference See the official MIDI 1.0 Control Change Messages Spec .","ref":"Midiex.Message.html#control_change/2"},{"type":"function","title":"Midiex.Message.note/1","doc":"Returns the MIDI numerical code for a note. Takes either a string or atom representation of a note as the first parameter. Example # Return the code for middle-C (also known as C4) Message . note ( :C4 ) Message . note ( &quot;C4&quot; ) Message . note ( :MiddleC ) Message . note ( &quot;MiddleC&quot; ) # These all return: 60 # Return the code for A-sharp 3 Message . note ( :As3 ) Message . note ( &quot;A#3&quot; ) # These both return: 58","ref":"Midiex.Message.html#note/1"},{"type":"function","title":"Midiex.Message.note_off/2","doc":"Creates a MIDI note-off message. Takes a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter. The following options can be passed: velocity: a number between 0 and 127 representing how hard (or loud) a key was pressed. By defaut 127 is used. channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example # Note-off for middle-C Message . note_off ( :C4 ) # Returns: &lt;&lt;128, 60, 127&gt;&gt;","ref":"Midiex.Message.html#note_off/2"},{"type":"function","title":"Midiex.Message.note_on/2","doc":"Creates a MIDI note-on message. Takes a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter. The following options can be passed: velocity: a number between 0 and 127 representing how hard (or loud) a key was pressed. By defaut 127 is used. channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Note that MIDI channels are in the range 0 - 15. But in MIDI software and hardware it may be offset by +1, so MIDI channel 0 might be called MIDI channel 1 and so on to channel 16. Examples # Note-on message for middle-C Midiex.Message . note_on ( :C4 ) # Returns: &lt;&lt;144, 60, 127&gt;&gt; # Note-on message for middle-C on channel 2 Midiex.Message . note_on ( :C4 , channel : 2 ) # Returns: &lt;&lt;146, 60, 127&gt;&gt; # Note-on message for middle-C on channel 2 with a velocity of 40 Midiex.Message . note_on ( :C4 , channel : 2 , velocity : 40 ) # Returns: &lt;&lt;146, 60, 40&gt;&gt; These can be sent to a connection using Midiex.send_msg/2 , for example: alias Midiex.Message Midiex . send_msg ( out_conn , Message . note_on ( :C4 , channel : 2 , velocity : 40 ) )","ref":"Midiex.Message.html#note_on/2"},{"type":"function","title":"Midiex.Message.pan/2","doc":"Change the panoramic (pan) of a channel. This shifts the sound from the left or right ear in when playing stereo. Values below 64 moves the sound to the left, and above to the right.","ref":"Midiex.Message.html#pan/2"},{"type":"function","title":"Midiex.Message.pitch_bend/3","doc":"Creates a pitch bend message, representing a change in pitch. Pitch bend change messages a usually sent from a keyboard with a pitch bend wheel. A pitch bend message includes two data bytes to specify the pitch bend value. Having two bites allows creates a higher resolution, making the pitch changes smoother.","ref":"Midiex.Message.html#pitch_bend/3"},{"type":"function","title":"Midiex.Message.polyphonic_aftertouch/2","doc":"Creates a polyphonic aftertouch message. On a keyboard, a polyphonic aftertouch message is sent by pressing down further on a key after it has already reached the bottom. Not all keyboards have aftertouch. Note: polyphonic_aftertouch is specific to each key, where as channel_aftertouch is average amount of pressure applied to whichever keys are held down. Takes a note as the first parameter and the following options: pressure: a number between 0 and 127 representing the pressure on the key. By defaut 127 is used. channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example alias Midiex.Message # Create a series of aftertouch messages from 0 to 127 for the note middle-C (C4) 0 .. 127 // 1 |&gt; Enum . map ( fn pressure -&gt; Message . polyphonic_aftertouch ( :C4 , pressure : pressure ) end )","ref":"Midiex.Message.html#polyphonic_aftertouch/2"},{"type":"function","title":"Midiex.Message.program_change/2","doc":"Creates a program change message, used select the instrument type to play sounds with.","ref":"Midiex.Message.html#program_change/2"},{"type":"function","title":"Midiex.Message.volume/2","doc":"","ref":"Midiex.Message.html#volume/2"},{"type":"module","title":"Midiex.MidiPort","doc":"A struct representing a MIDI port. The keys are as follows: direction which is an atom of value :input or :output (for input or output port) name which is a string the backend reported as the name of the port. With MIDI hardware, this is often the name of the device. num an integer index representing the port starting at 0. Both input and output ports will start with 0. port_ref a reference (e.g. #Reference&lt;0.2239960018.1937899544.176288&gt; ) to the port object in midir (Rust). Notes from midir How a port is identified internally is backend-dependent. If the backend allows it, port objects remain valid when other ports in the system change (i.e. it is not just an index). MidiInputPort: https://docs.rs/midir/latest/midir/struct.MidiInputPort.html MidiOutputPort: https://docs.rs/midir/latest/midir/struct.MidiInputPort.html Example Midiex . ports ( ) This will return MIDI ports available on your system, for example: [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.2239960018 . 1937899544.176288 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.2239960018 . 1937899544.176289 &gt; } ]","ref":"Midiex.MidiPort.html"},{"type":"module","title":"Midiex.OutConn","doc":"A struct representing an open connection to a MIDI output port. The keys are as follows: conn_ref the reference (e.g. #Reference&lt;0.2239960018.1937899544.176288&gt; ) to the connection object in midir (Rust). name a string containing the name of the port this connection is to port_num a integer representing the index of the output port. Documentation from midir See MidiOutputConnection at: https://docs.rs/midir/latest/midir/struct.MidiOutputConnection.html Example # Pass a port from taken from Midiex.ports(:output) # e.g. port = Midiex.ports(:output) |&gt; List.first() port = % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.3876911033 . 1674706968.249863 &gt; } output_conn = Midiex . open ( port ) output_conn will look something like this: % Midiex.OutConn { conn_ref : # Reference &lt; 0.3876911033 . 1674706945.249916 &gt; , name : &quot;IAC Driver Bus 1&quot; , port_num : 0 } An output port can be closed as follows: Midiex . close ( output_conn ) # :ok is returned if successful","ref":"Midiex.OutConn.html"},{"type":"module","title":"Midiex.VirtualMidiPort","doc":"A struct representing a virtual MIDI port.","ref":"Midiex.VirtualMidiPort.html"},{"type":"extras","title":"README","doc":"Midiex overview Midiex is a cross-platform, real-time MIDI processing library in Elixir.","ref":"readme.html"},{"type":"extras","title":"README - midir Rust library","doc":"Using Rustler , Midiex wraps the excellent midir Rust library. Midir support a range of platforms and backends, such as: ALSA (Linux) WinMM (Windows) CoreMIDI (MacOS, iOS) WinRT (Windows 8+), Jack (Linux, macOS), Using WinRT or Jack requires special feature flags enabled. See the midir GitHub and create docs for more details.","ref":"readme.html#midir-rust-library"},{"type":"extras","title":"README - Status","doc":"This library is currently under active development and it’s API is likely to change. It's been tested on MacOS only.","ref":"readme.html#status"},{"type":"extras","title":"README - API","doc":"At it's most basic level, the core functions of Midiex are for: listing or counting MIDI ports availble (for example, a keyboard or synth) opening or closing connections to MIDI ports sending or receiving messages to and from connections creating a virtual output connection so your Elixir application appears as a MIDI input port on the host OS. creating a virtual input connection so your Elixir application appears as a MIDI output port on the host OS.","ref":"readme.html#api"},{"type":"extras","title":"README - Feature support","doc":"Not all midir features have been wrapped and some features are backend specific: Virtual output connection : currently on every platform but Windows Virtual input connection : currently on every platform but Windows.","ref":"readme.html#feature-support"},{"type":"extras","title":"README - MIDI messages","doc":"MIDI messages are in binary format. They're usually in the format of one status byte followed by one or two data bytes. For example, the status byte for 'Note On' is 0x90 in HEX format. The data byte representing the note Middle C is 60 . The data byte representing velocity (i.e. how hard the key was struck when the note was played) is an integer in the range 0 - 127 where 127 is the loudest. Putting that together, the message to play Middle C at a velocity of 127 is: &lt;&lt;0x90, 60, 127&gt;&gt; You can stop the same note from playing by sending the 'Note Off' status byte 0x80 , which would make the message: &lt;&lt;0x80, 60, 127&gt;&gt; . For more information on MIDI messages, see the offical MIDI Assocations Specifications , Expanded MIDI 1.0 message list or the various articles online such as this one .","ref":"readme.html#midi-messages"},{"type":"extras","title":"README - Example","doc":"# List MIDI ports Midiex . ports ( ) # Create a virtual output connection piano = Midiex . create_virtual_output ( &quot;piano&quot; ) # Returns an output connection: # %Midiex.OutConn{ # conn_ref: #Reference&lt;0.1633267383.3718381569.210768&gt;, # name: &quot;piano&quot;, # port_num: 0 # } # Send to MIDI messages to a connection note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off )","ref":"readme.html#example"},{"type":"extras","title":"README - Getting started","doc":"Adding it to your Elixir project The package can be installed by adding midiex to your list of dependencies in mix.exs: def deps do [ { :midiex , &quot;~&gt; 0.1.1&quot; } ] End Using within LiveBook and IEx Mix . install ( [ { :midiex , &quot;~&gt; 0.1.1&quot; } ] ) LiveBook tour Also see the introductory tour in LiveBook at /livebook/midiex_notebook.livemd .","ref":"readme.html#getting-started"},{"type":"extras","title":"README - Documentation","doc":"The docs can be found at https://hexdocs.pm/midiex .","ref":"readme.html#documentation"},{"type":"extras","title":"MIDIex notebook","doc":"Mix . install ( [ { :midiex , path : &quot;/Users/haubie/Development/midiex&quot; } ] ) == &gt; midiex Compiling 1 file ( . ex ) Compiling crate midiex in release mode ( native / midiex ) Compiling midiex v0 . 1.0 ( / Users / haubie / Development / midiex / native / midiex ) Finished release [ optimized ] target ( s ) in 0.55 s :ok","ref":"midiex_notebook.html"},{"type":"extras","title":"MIDIex notebook - Setup","doc":"","ref":"midiex_notebook.html#setup"},{"type":"extras","title":"MIDIex notebook - Finding devices (ports)","doc":"How many ports are there? Midiex . port_count ( ) %{ input : 4 , output : 4 } List devices (ports) ports = Midiex . ports ( ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215786 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215787 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215788 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215789 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215790 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215791 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215792 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215793 &gt; } ] Filter to show input or output ports Midiex . ports ( :output ) [ % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215798 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215799 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215800 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215801 &gt; } ] out_port = Midiex . ports ( &quot;Arturia&quot; , :output ) |&gt; List . first ( ) % Midiex.MidiPort { direction : :output , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215807 &gt; } out_conn = Midiex . open ( out_port ) % Midiex.OutConn { conn_ref : # Reference &lt; 0.522475107 . 1026162689.216660 &gt; , name : &quot;Arturia MicroFreak&quot; , port_num : 1 } Midiex . send_msg ( out_conn , &lt;&lt; 0x90 , 60 , 127 &gt;&gt; ) % Midiex.OutConn { conn_ref : # Reference &lt; 0.522475107 . 1026162689.216660 &gt; , name : &quot;Arturia MicroFreak&quot; , port_num : 1 } Midiex . ports ( :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215810 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215811 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215812 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215813 &gt; } ] Filter by port name Midiex . ports ( &quot;IAC&quot; , :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215818 &gt; } ]","ref":"midiex_notebook.html#finding-devices-ports"},{"type":"extras","title":"MIDIex notebook - Virtual devices","doc":"virtual_conn = Midiex . create_virtual_output ( &quot;My Virtual Connection&quot; ) % Midiex.OutConn { conn_ref : # Reference &lt; 0.522475107 . 1026162689.216746 &gt; , name : &quot;My Virtual Connection&quot; , port_num : 4 } Note that although you've created this virtual output, on your system it will appear as an import port to be discoverable by other MIDI software or devices. If you call Midiex.ports/1 you'll see it as an input: ports = Midiex . ports ( :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215826 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215827 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215828 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215829 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Virtual Connection&quot; , num : 4 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215830 &gt; } ]","ref":"midiex_notebook.html#virtual-devices"},{"type":"extras","title":"MIDIex notebook - Make a connection to a device","doc":"Connect to an output device and send a message out_conn = Midiex . ports ( &quot;IAC&quot; , :output ) |&gt; List . first ( ) |&gt; Midiex . open ( ) % Midiex.OutConn { conn_ref : # Reference &lt; 0.522475107 . 1026162689.216766 &gt; , name : &quot;IAC Driver Bus 1&quot; , port_num : 0 } Midiex.Backend . get_subscribed_ports ( ) [ ] Midiex.Backend . listen ( Midiex . ports ( :input ) |&gt; Enum . at ( 2 ) ) [ % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215846 &gt; } ] in_port = Midiex . ports ( &quot;Key&quot; , :input ) |&gt; List . first ( ) % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215855 &gt; } 1 .. 10 // 1 |&gt; Enum . map ( fn i -&gt; :timer . sleep ( 1000 ) Midiex.Backend . test ( in_port ) end ) warning : variable &quot;i&quot; is unused ( if the variable is not meant to be used , prefix it with an underscore ) livebook / midiex_notebook . livemd #cell:7euaccszbergu2jit2isskr7ejpvc65q:2 [ :ok , :ok , :ok , :ok , :ok , :ok , :ok , :ok , :ok , :ok ] # Midiex.Backend.clear_subscribed_ports() nil Connect and listen to an input device defmodule MyMidiServer do use GenServer @impl true def init ( state \\\\ [ ] ) do IO . inspect ( state , label : &quot;INIT&quot; ) { :ok , state } end @impl true def handle_info ( msg , state ) do case msg do msg -&gt; IO . inspect ( msg , label : &quot;FROM RUST&quot; ) msg _ -&gt; msg end { :noreply , state } end def listen_to ( device_name \\\\ &quot;IAC Driver&quot; ) do Midiex . ports ( ) |&gt; Midiex . filter_port_name_contains ( device_name , direction : :input ) |&gt; List . first ( ) |&gt; Midiex . listen ( ) end def listen ( pid , device_name ) do GenServer . cast ( pid , { :listen , device_name } ) end @impl true def handle_cast ( { :listen , device_name } , state ) do listen_to ( device_name ) { :noreply , state } end def subscribe ( pid ) do GenServer . cast ( pid , :subscribe ) end @impl true def handle_cast ( :subscribe , state ) do Midiex . subscribe ( ) { :noreply , state } end end warning : clauses with the same name and arity ( number of arguments ) should be grouped together , &quot;def handle_cast/2&quot; was previously defined ( livebook / midiex_notebook . livemd #cell:7rfguhfubcggnj6q6hoiiqg7lbqbjlab:36) livebook / midiex_notebook . livemd #cell:7rfguhfubcggnj6q6hoiiqg7lbqbjlab:46 warning : this clause cannot match because a previous clause at line 13 always matches livebook / midiex_notebook . livemd #cell:7rfguhfubcggnj6q6hoiiqg7lbqbjlab:17 warning : Midiex . filter_port_name_contains / 3 is undefined or private livebook / midiex_notebook . livemd #cell:7rfguhfubcggnj6q6hoiiqg7lbqbjlab:26: MyMidiServer.listen_to/1 { :module , MyMidiServer , &lt;&lt; 70 , 79 , 82 , 49 , 0 , 0 , 21 , ... &gt;&gt; , { :handle_cast , 2 } } { :ok , pid } = GenServer . start_link ( MyMidiServer , [ ] ) INIT : [ ] { :ok , # PID &lt; 0 . 264 . 0 &gt; } # MyMidiServer.listen(pid, &quot;KeyStep&quot;) # MyMidiServer.listen(pid, &quot;MicroFreak&quot;) nil MyMidiServer . subscribe ( pid ) :ok FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } Midiex.Backend . listen ( Midiex . ports ( :input ) |&gt; Enum . at ( 1 ) ) [ % Midiex.MidiPort { direction : :input , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215863 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.522475107 . 1026162712.215846 &gt; } ] FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;Arturia MicroFreak&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;KeyStep Pro&quot; , &quot;port_num&quot; =&gt; 2 } Midiex.Backend . get_subscribed_ports ( ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.2084832073 . 4240834584.3914 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Virtual Connection&quot; , num : 1 , port_ref : # Reference &lt; 0.2084832073 . 4240834584.3918 &gt; } ] FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;IAC Driver Bus 1&quot; , &quot;port_num&quot; =&gt; 0 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;My Virtual Connection&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;IAC Driver Bus 1&quot; , &quot;port_num&quot; =&gt; 0 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;My Virtual Connection&quot; , &quot;port_num&quot; =&gt; 1 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;IAC Driver Bus 1&quot; , &quot;port_num&quot; =&gt; 0 } FROM RUST : %{ &quot;port_name&quot; =&gt; &quot;My Virtual Connection&quot; , &quot;port_num&quot; =&gt; 1 } Midiex.Backend . clear_subscribed_ports ( ) [ ]","ref":"midiex_notebook.html#make-a-connection-to-a-device"},{"type":"extras","title":"License (MIT)","doc":"MIT License Copyright (c) 2023 David Haubenschild. Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}]